#include "IcomMessage.h"

// constructor
IcomMessage::IcomMessage(void) :
m_Counter(0)
{
};


// Also check to see if each frequency element is a valid BCD
boolean
IcomMessage::isFreq(void) const
{
	if (m_Counter != 10) return false;
	if ( ! (buffer[3] == 3  || !buffer[3] || buffer[3] == 5) ) return false;
	const byte * cp = &buffer[8];
	const byte * end = &buffer[4];
	for ( ; cp >= end ; --cp)
	{
		if ( (*cp & 0x0f) > 9) return false;
		if ( ( (*cp >> 4) & 0x0f) > 9) return false;
	}
	return true;
}


unsigned long 
IcomMessage::getFreq(void) const
{
	unsigned long ret = 0;
	const byte * cp = &buffer[8];
	const byte * end = &buffer[4];
	for ( ; cp >= end ; --cp)
	{
		ret *= 100;
		ret += static_cast<unsigned long> (bcdByte(*cp));
	}
	return ret;
}

// put a byte into the message,  but first check to see if there is space
boolean
IcomMessage::add(const byte x)
{
	if (m_Counter >= icomMessageMaxLength)
	{
		setMTY();
		return false;
	}
	buffer[m_Counter] = x;
	++m_Counter;
	return true;    
}


void
IcomMessage::dump(void) const
{

}

//  Does sanity checks on a message to see if it follows the rules
boolean
IcomMessage::ValidateReceiveMessage(void)
{
	// any valid ICOM message must be at least 5 bytes bytes long
	if (m_Counter < 5) return false;
	// check the range of the source ID
	byte a = getSourceID();
	if ( ! (a < 0x80 || isControllerID(a) ) ) return false;
	// check the range of the destination ID
	a = getDestinationID();
	if ( ! (a < 0x80 || isControllerID(a) ) )  return false;
	// check the range of the commands
	a = getCommand();
	if ( ! (a < 0x20 || a == 0xfa || a == 0xfb)) return false;
	// check the length of various commands
	switch (a)
	{
	case 5:  // writes operating frequency
	case 0:  // Transfers operating frequency data
		if (m_Counter != 10) return false;
		break;
	case 1:  // Transfer operating mode data
		if (m_Counter != 7) return false;
		break;
	case 3:  // Read out operating frequency data  COMMON!!
		if (! (m_Counter == 5 || m_Counter == 10)) return false;
		break;
	case 4: // Operating mode and IF passband bandwidth  COMMON!
		if (! (m_Counter == 5 || m_Counter == 7)) return false;
		break;
	case 6: // Write Operating mode and IF pass bandwidh
		if (m_Counter != 7) return false;
		break;
	default:
		break;
	}
	if (a == 4 && m_Counter == 7)
	{
		if (getMode() > 10) return false;
		if (getBW() > 6) return false;
	}
	return true;  
}


void
IcomMessage::setSourceDestinationID(const byte source, const byte dest)
{
	buffer[0] = icomMessageStartByte;
	buffer[1] = dest;
	buffer[2] = source;
}

void
IcomMessage::setModeBW(const byte mode, const byte bw)
{
	m_Counter = 7;
	buffer[3] = 4; // Mode BW command
	buffer[4] = mode;
	buffer[5] = bw;
	buffer[6] = icomMessageStopByte;
}

void
IcomMessage::setFreq(const unsigned long freq)
{
	m_Counter = 10;
	buffer[3] = 3; // Frequency mode
	unsigned long f =  (freq);
	for (byte i = 0 ; i < 5 ; ++i)
	{
		unsigned long x = f % 100;
		buffer[4+i] = bcdByteEncode( static_cast<byte>(x) );
		f = f / 100;
	}
	buffer[9] = icomMessageStopByte;
}


void 
IcomMessage::setFreqCmd0(const unsigned long freq)
{
	setFreq(freq);
	buffer[3] = 0;  // set command to zero
}

void
IcomMessage::setModeBWCmd1(const byte mode, const byte bw)
{
	setModeBW(mode,bw);
	buffer[3] = 1;  // set command as 1
}

boolean
IcomMessage::isEqual(const IcomMessage & x)
{
	if (x.m_Counter != m_Counter) return false;
	for (byte i = 0 ; i < m_Counter ; ++i)
		if (x.buffer[i] != buffer[i]) return false;
	return true;
}

void
IcomMessage::extendlongby(const byte x)
{
	for (byte i = 0 ; i < x ; ++i)
		buffer[i+5] = '\0';
	m_Counter += x;
	buffer[5+x] = icomMessageStopByte;
}

void
IcomMessage::copy(const IcomMessage &a)
{
	m_Counter = a.m_Counter;
	::memcpy(buffer,a.buffer,icomMessageMaxLength);
}