
#ifndef ARDUIONO_ICOMMESSAGE
#define ARDUIONO_ICOMMESSAGE

#include <Arduino.h>

// What is the max lengh of an ICOM Message  Should be 14 but 16 gives some protection
static const byte icomMessageMaxLength = 16;
/// What is the value of the first two bytes of an Icom Message (only one stored in this class)
static const byte icomMessageStartByte = 0xFE;
///  What is the value of the last byte of an Icom Message
static const byte icomMessageStopByte  = 0xFD;

/// determines if an ID is a controller or not...
inline bool isControllerID(const byte x) { return ( (x & 0xf0) == 0xe0 ); }
/**
* @author James Michener K9JM  (c) 2012
*
* Routine that is a container class and interprets / makes
* IcomMessages
* Only the first of the two icomMessageStartBytes are stored
* in this message
*/


class IcomMessage
{
public:
	IcomMessage(void);
	~IcomMessage(void) {};
	/// add byte to buffer  return true if ok
	boolean add(const byte x);
	inline byte * get(void) { return buffer; }
	inline byte getLen(void) { return m_Counter; }
	inline void setInUse(void)  { m_Counter = 0xff; }
	inline boolean isMarkedInUse(void) { return m_Counter == 0xff; }
	inline boolean isEmpty(void) { return !m_Counter; }
	inline void setMTY(void) { m_Counter = 0; }
	inline byte getSourceID(void) const { return buffer[2]; }
	inline byte getDestinationID(void) const { return buffer[1]; }
	inline byte getCommand(void) const { return buffer[3]; }

	inline boolean isModeBW(void) const { return  m_Counter == 7  && (buffer[3] == 4 || buffer[3] == 1  || buffer[3] == 6) ; }
	inline boolean getMode(void)  const { return buffer[4]; }
	inline boolean getBW(void)    const { return buffer[5]; }

	// tests to see if special okay or 'no good' message
	inline boolean isOkay(void)   const { return (m_Counter == 5 && buffer[3] == 0xfb); }
	inline boolean isNG(void)     const { return (m_Counter == 5 && buffer[3] == 0xfa); }

	inline boolean isFreqQuery(void) const { return  (m_Counter == 5 && buffer[3] == 3) ; }
	inline boolean isModeBWQuery(void)  const { return  m_Counter == 5  && buffer[3] == 4 ; }
	inline boolean isVolumeQuery(void)  const { return  m_Counter == 6 && buffer[3] == 0x14 && buffer[4] == 1; }
	inline boolean isSpecialQuery(void) const { return  m_Counter == 6 && buffer[3] == 0x1a && buffer[4] == 6; }

	inline boolean isChangeFreqOrModeCommand(void) const { return (buffer[3] == 7 || buffer[3] == 8); }
	inline boolean isChangeFreqCommand(void) const { return (buffer[3] == 5); }
	inline boolean isChangeModeCommand(void) const { return (buffer[3] == 6); }

	inline boolean isVFOCommand(void) const { return (buffer[3] == 7); }
	inline boolean isVFOSubCommand(void) const  { return ( buffer[4] == 0xd1); } 
	inline boolean isVFOMainCommand(void) const { return ( buffer[4] == 0xd0); } 

	void extendlongby(const byte x);

	boolean isFreq(void) const;
	unsigned long getFreq(void) const;
	/// return false if there is something obviously wrong with it
	/// does sanity checks.  Wish here was a check sum or CRC!!
	boolean ValidateReceiveMessage(void);

	boolean isEqual(const IcomMessage & x);

	// Routines for building an icomMessage

	inline void swapSourceDestinationID(void) { byte t = buffer[1]; buffer[1] = buffer[2]; buffer[2] = t; }
	void setSourceDestinationID(const byte source, const byte dest);
	void setModeBW(const byte mode, const byte bw);
	void setFreq(const unsigned long freq);
	void setFreqCmd0(const unsigned long freq);
	void setModeBWCmd1(const byte mode, const byte bw);

	void copy(const IcomMessage & a);

	void dump(void) const;

	// DATA

	// countes the bytes as they come in
	// otherwise is the byte count in message
	// if zero them mty,  if 0xff then in use but no data.
	byte m_Counter;  

	// The actual message
	byte buffer[icomMessageMaxLength];

private:

	// used by getFreq()
	inline byte bcdByte(const byte x) const { return  (((x & 0xf0) >> 4) * 10) + (x & 0x0f); } 
	// used by setFreq()
	// input is between 0 and 99 decimal.  output 0 to 0x99
	inline byte bcdByteEncode(const byte x) const { return ((x / 10) << 4)  +  (x % 10); } 


};

#endif // ARDUIONO_ICOMMESSAGE
