#ifndef ARDUINO_IcomSerialRX
#define ARDUINO_IcomSerialRX

#include <Arduino.h>

#include "IcomMessage.h"
#include "SimpleTimeout.h"

const unsigned long icomMessageReceiveTimeOutinMS = 120;
const unsigned long icomMessageTransmitTimeOutinMS = 170;

/*
*  Constants to define the IcomSerialRXFlags
*/
/// on receive has a valid start character been received
const byte isValidStartFlag = 0;
/// is the serial port full duplex (true) or simplex (false)
const byte isDuplexFlag = 1;
/// am actually sending a message
const byte isTransmittingFlag = 2;
/// are we waiting after RX data before sending?
const byte isTXHoldOff = 3;
/// is this a new freq?
//const byte isNewFreq = 4;
/// has a radio on this port?
const byte hasRadioOnPort  = 5;


class IcomSerialRX
{
public:
	IcomSerialRX(void) :
		flags(0),
		m_lastin(0)
	{ 
		stReceive.start(icomMessageReceiveTimeOutinMS );
	}
	~IcomSerialRX(void)
	{
	}

	// Check for serial port for received messages
	void checkReceive(void);


	/**
	* @return 
	*  0 -  nothing to report
	*  1 -  Message timed out
	*  2 -  Message too long
	*  3 -  Message Received
	*  4 -  End of Message byte without a valid start
	*/
	byte add(const byte x);

	// what is the current message that we are taking data for
	IcomMessage imReceive;

	inline void flagSet(const byte x) { bitSet(flags,x); }
	inline void flagClear(const byte x) { bitClear(flags,x); }
	inline bool flagRead(const byte x) const { return bitRead(flags,x); }


private:
	SimpleTimeout stReceive;
	// have we received a valid start during receive
	byte flags;
	// last character received.  Used to detect the start sequence.
	byte m_lastin;
};

#endif // ARDUINO_IcomSerialRX

