#include <avr/pgmspace.h>
#include <Arduino.h>

//# Workaround for http://gcc.gnu.org/bugzilla/show_bug.cgi?id=34734
#ifdef PROGMEM
#undef PROGMEM
#define PROGMEM __attribute__((section(".progmem.data")))
#endif

PROGMEM prog_uint16_t FreqTuningTable[] = { 
    //     47    48   49   50   51   52    53     54    53   52    51   50      49    48    47    46    45    44    43    43    41    40    39    38   37     36    35   34    33     32
	0 , 3500, 3510, 3520,3545,3590, 3640, 3670, 3710, 3730, 3770, 3790, 3807, 3819, 3823, 3830, 3838,3843, 3860, 3870, 3885, 3895, 3905 ,3920, 3932, 3945, 3955, 3963, 3971, 3985, 3990, 4000, 0xffff
};

// for each band of frequencies (see FreqTuningTable above ) what is the corresponding Tuning setting on the Matchbox.
PROGMEM prog_int8_t TuningTable[] = {
	0xff, 47, 48,49,50, 51, 52, 53, 54,53,52,51,50,49,48,47,46,45,44,43,42,41,40,39,38,37,36,35,34,32, 0xff, 0xff, 0xff
};


PROGMEM prog_uint16_t FreqMatchingTable[] = { 
    //    34   35   36   37   38   39   40   41    42   43    44    45    46    47    48    49   50    51    52    53     54    55    56    57   58    59     60    61    62    63    64    65    66    67     68    69    70    71    72    73   74     75    76    77    78    79   80     81    82     83    84    85    86    87   88     89
	0 , 3500,3520,3530,3540,3555,3563,3573,3580,3590, 3600, 3610, 3620, 3630, 3640, 3660, 3670, 3680, 3687, 3695, 3712, 3721, 3730, 3737, 3745, 3755, 3765, 3773, 3780, 3787, 3797, 3805, 3810, 3815,  3820, 3823, 3827, 3835, 3846, 3855, 3858, 3865, 3872, 3880, 3887, 3895, 3906, 3915, 3921, 3930,  3937, 3947, 3955, 3965, 3972, 3985, 3995, 4000, 0xffff
};

// for each band of frequencies (see FreqTuningTable above ) what is the corresponding Tuning setting on the Matchbox.
PROGMEM prog_int8_t MatchingTable[] = {
	0xff, 34 , 35 , 36 , 37,38,39,40, 41,42, 43, 44, 45, 46, 47, 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89, 0xff, 0xff
};

static uint8_t lastTuningGuess = 0xff; 
static uint8_t lastMatchingGuess = 0xff; 

byte
lookupTable ( const unsigned int value,  prog_uint16_t * table,const uint8_t lastGuess,const  uint16_t tableSize )
{
	int bRet = (lastGuess != 0xff) ? lastGuess : tableSize / (sizeof(prog_uint16_t) * 2);
	int lowIndex = 0;
	int highIndex = tableSize / sizeof(prog_uint16_t);

	while (true)
	{
		unsigned int tableValueHigh = pgm_read_word( table + bRet + 1);
		unsigned int tableValueLow = pgm_read_word( table + bRet);
		if (tableValueLow > value )
			highIndex = bRet;
		else if ( tableValueHigh <= value )
			lowIndex = bRet + 1;
		else
			break;
		bRet = (highIndex + lowIndex) / 2;
		continue; 
	}
	return static_cast<byte>(bRet);
}


byte
freqToTuning(const unsigned long freq)
{
	lastTuningGuess = lookupTable( static_cast<unsigned int>(freq/1000), FreqTuningTable,lastTuningGuess,sizeof(FreqTuningTable));
    if (lastTuningGuess >= (sizeof(FreqTuningTable) / sizeof(prog_uint16_t)) -1 )
        return 0xff;
    return pgm_read_byte(TuningTable+lastTuningGuess);
}

byte
freqToMatching(const unsigned long freq)
{
	lastMatchingGuess = lookupTable( static_cast<unsigned int>(freq/1000), FreqMatchingTable,lastMatchingGuess,sizeof(FreqMatchingTable));
     if (lastTuningGuess >= (sizeof(FreqMatchingTable) / sizeof(prog_uint16_t)) -1 )
         return 0xff;
   return pgm_read_byte(MatchingTable+lastMatchingGuess);
}
