
#include <LiquidCrystal.h>
#include <DFR_Key.h>
#include <Wire.h>
#include "StepperMotor.h"
#include "IcomSerialRX.h"
#include "MatchboxMap.h"

//Pin assignments for DFRobot LCD Keypad Shield
LiquidCrystal lcd(8, 9, 4, 5, 6, 7); 
//---------------------------------------------

/**  Uses the DFR LCD Keyboard **/

DFR_Key keypad;

/*
IcomSerialRX is a class that reads messages from the CI-V and reports frequency
*/
IcomSerialRX icom;

uint8_t gMotor;

StepperMotor tuning(2,3,4,0,3);
StepperMotor matching(5,7,6,1,2);

void sendMotor(void)
{
	Wire.beginTransmission(0x20);  // I2C address
	Wire.write(gMotor);
	Wire.endTransmission(false);
}

int16_t lastA = 0x7fff;
int16_t lastB = 0x7fff;

void lcdStatus(const int16_t a, const int16_t b)
{
	if (a == lastA && b == lastB) return;
	lcd.setCursor(0,1);
	lcd.print(F("Tun "));
	lcd.print(a);
	lcd.print(F(" - Mat "));
	lcd.print(b);
	lcd.print(F("   "));
}


void setup()
{
	Serial.begin(9600);  // Set CI-V Baudrate
	lcd.begin(16, 2);
	lcd.clear();
	lcd.noCursor();
	lcd.home();
	lcd.print(F("MATCHBOX - K9JM"));
	lcd.setCursor(0,1);
	lcd.print(F("Aug 3 2012"));
	Wire.begin();
	tuning.startUp();
	matching.startUp();
	delay(500);
	/* add setup code here */
	tuning.home();
	matching.home();
	unsigned long timeEnd = millis() + 5000;
	while (tuning.getState() != StepperMotor::E_IDLE ||
		matching.getState() != StepperMotor::E_IDLE)
	{
		tuning.evaluate();
		matching.evaluate();
		if (millis() > timeEnd) break;
	}
	lcd.setCursor(0,1);
	lcd.print(F("READY      "));
	delay(100);

}

int16_t last = 0;
uint8_t sameCount = 0;

const uint8_t stepSize = 1;

void loop()
{
	tuning.evaluate();
	icom.checkReceive();
	matching.evaluate();
	icom.checkReceive();

	int16_t localKey = keypad.getKey();
	if (localKey != SAMPLE_WAIT)
	{
		/*
		lcd.clear();
		lcd.setCursor(0,0);
		lcd.print(sameCount);
		lcd.print(" ");
		lcd.print(last);
		lcd.print(" ");
		lcd.print(localKey);
		delay(20);
		*/
		if (localKey != last)
		{
			last = localKey;
			sameCount = 0;
		}
		else
		{
			if (sameCount < 25)
			{
				sameCount++;
			}
		}
		//		if (sameCount == 20)
		{
            tuning.setDesiredPosition(DoNothingDesiredPosition);
            matching.setDesiredPosition(DoNothingDesiredPosition);
			switch(last)
			{
			case 1:
				if (tuning.getCount()) break;
				tuning.incPosition(stepSize);
				lcd.clear();
				lcd.home();
				lcd.print(F("TUNING +"));
				lcd.print(stepSize);
				break;
				break;
			case 2:
				if (tuning.getCount()) break;
				tuning.incPosition(-stepSize);
				lcd.clear();
				lcd.home();
				lcd.print(F("TUNING -"));
				lcd.print(stepSize);
				break;
			case 3:
				if (matching.getCount()) break;
				matching.incPosition(stepSize);
				lcd.clear();
				lcd.home();;
				lcd.print(F("MATCHING +"));
				lcd.print(stepSize);
				break;
			case 4:
				if (matching.getCount()) break;
				matching.incPosition(-stepSize);
				lcd.clear();
				lcd.home();
				lcd.print(F("MATCHING -"));
				lcd.print(stepSize);
				break;
			case 5:
				matching.incPosition(10);
				tuning.incPosition(10);
				lcd.clear();
				lcd.home();
				lcd.print(F("INCREMENT 10"));
				break;
			default:
				break;
			}
		}
	}
    icom.checkReceive();

	lcdStatus(tuning.getPosition(), matching.getPosition());
	//    torchure();
}



void onMessageGlobal(IcomMessage * msg)
{
	if (!msg->isFreq()) return;
	lcd.clear();
	lcd.home();
	unsigned long freq = msg->getFreq();  
    lcd.print(freq/1000);
    uint16_t res = freq%1000;
    lcd.print(F("."));
    uint16_t a1 = res % 1000;
    lcd.print(a1/100,DEC);
    a1 = res % 100;
    lcd.print(a1/10,DEC);
    lcd.print(F("KHz"));
#if 1
	uint8_t tuningValue = freqToTuning(freq);
	uint8_t matchingValue =  freqToMatching(freq);
#else
	uint8_t tuningValue = (freq / 1000) % 100;
	uint8_t matchingValue =  100 - ((freq / 1000) % 100);
#endif

	if (tuningValue == 0xff )
		tuning.setDesiredPosition(DoNothingDesiredPosition);
	else
		tuning.setDesiredPosition( static_cast<int16_t>(tuningValue));

	if (matchingValue == 0xff )
		matching.setDesiredPosition(DoNothingDesiredPosition);
	else
		matching.setDesiredPosition( static_cast<int16_t>(matchingValue));
}