#ifndef __STEPPERMOTOR_H_
#define __STEPPERMOTOR_H_

#include <Arduino.h>

const int16_t DoNothingDesiredPosition  = 0x7fff;

class StepperMotor
{
public:
	enum motorState 
	{
		E_IDLE = 0,
		E_START,
		E_MOVE,
		E_STOP,
		E_HOME

	};

	StepperMotor(const uint8_t enable, const uint8_t a, const uint8_t b,
        const uint8_t led, const uint8_t opto);
	
	~StepperMotor(void) {}
	// Command move relative
	inline void incPosition(const int16_t x) { count += x; evaluate(); }
	// This is a command...
    // go to a absolute position ... only valid after a home has been done.
	inline void gotoPosition(const int16_t x) { incPosition( x - (position + count)); }

	inline int16_t getPosition(void) const { return position; }

	// called in the main loop to evaluate a change in the position
	// of the motor.
	void evaluate(void);
	
	// drop whatever you're doing and go to home.
	void home(void);
    /**
     * When the motor is idle,  it will go to this position
     * This is the best place to tell it where to go
     * @param pos   if 0xff then it won't go anywhere
     */
    inline void setDesiredPosition(const int16_t pos) { desiredPosition = pos; }
    inline void incDesiredPosition(const int16_t delta) { desiredPosition += delta; }
    inline void startUp(void) { count = 0; goToIdle(); }

    inline motorState getState(void) const { return state; }
    inline int16_t getCount(void) const { return count; }
    inline uint8_t getPhase(void) const { return phase; }


private:

	void goToIdle(void);
	void goToStart(void);
	void setMotorBitsFromPhase(void);
	void setMotorBitsFromPhaseGo(void); 
	void stepPlus(const uint8_t delay);
	void stepMinus(const uint8_t delay);
	void stepCommon(const uint8_t delay);

	unsigned long timeNextStateChange;
	uint8_t bitEnable;
	uint8_t bitA;
	uint8_t bitB;
	uint8_t bitLEDPwr;
	uint8_t pinOpto;
	uint8_t phase;  // four phase counter
    int16_t desiredPosition;  // where it should go  0xff means no desired position
	int16_t count;  // how many places does it need to turn...
	int16_t position; ///  where it is w.r.t. home
	uint16_t homeCounter;  // counts number of positions stepped to find home
	bool bHomeHigh;
	motorState state;


};

#endif // __STEPPERMOTOR_H_
